//--------------------------------------------------------------------- 
#include "BaseApp.h"
//--------------------------------------------------------------------- 
BaseApp::BaseApp() 
    :BApplication(APPLICATION_SIGNATURE) 
{ 
    BResources res;
    BFile file;

    usedjapanese=false;
    if(file.SetTo(RESOURCEFILEPATH, B_READ_ONLY)==B_OK)
    {
        size_t siz;
        int32 *wk;
            
        res.SetTo(&file);
            
        siz=sizeof(int32);
        wk=(int32 *)res.LoadResource(B_INT32_TYPE,R_JAPANESE,&siz);
        if(wk!=NULL)
        	usedjapanese=(bool)*wk;

        file.Unset();    
    }

    EditorList=new BList(); 
    open_filepanel= new BFilePanel(); 
    findword="";
    replaceword="";
    ignorecase=false;
} 
//--------------------------------------------------------------------- 
BaseApp::~BaseApp() 
{ 
	BResources res;
    BFile file(RESOURCEFILEPATH, B_READ_WRITE | B_CREATE_FILE);
    int32 wk;
    
    res.SetTo(&file);

    wk=(int32)usedjapanese;
    res.AddResource(B_INT32_TYPE,R_JAPANESE,&wk,sizeof(wk));

    res.Sync();
    file.Unset();    

    delete open_filepanel; 
        
    for(int i=EditorList->CountItems()-1;i>=0;i--) 
    { 
        BAppMainWindow *targetwnd= 
                        ((BAppMainWindow *)EditorList->ItemAt(i)); 
        targetwnd->Lock();
        targetwnd->Quit(); 
        EditorList->RemoveItem(targetwnd); 
    } 
    delete EditorList; 
} 
//--------------------------------------------------------------------- 
BAppMainWindow *BaseApp::AddEditor(void) 
{ 
    BAppMainWindow *mainwnd; 
    BAppMainWindow *targetwnd=NULL;
    BRect wndpos(MAINWINDOW_POSITION_LEFT,MAINWINDOW_POSITION_TOP, 
                 MAINWINDOW_POSITION_WIDTH+MAINWINDOW_POSITION_LEFT-1, 
                 MAINWINDOW_POSITION_HEIGHT+MAINWINDOW_POSITION_TOP-1);
    BScreen mainscreen;
    float width,height;
    
    width=wndpos.Width();
    height=wndpos.Height();
    
    if(EditorList->CountItems()>0)
        targetwnd=(BAppMainWindow *)EditorList->ItemAt(
                                        EditorList->CountItems()-1);

    if(targetwnd!=NULL)
        wndpos.Set(targetwnd->Frame().left+B_V_SCROLL_BAR_WIDTH,
                   targetwnd->Frame().top+B_H_SCROLL_BAR_HEIGHT,
                   targetwnd->Frame().right+B_V_SCROLL_BAR_WIDTH,
                   targetwnd->Frame().bottom+B_H_SCROLL_BAR_HEIGHT);
    else
    {
        BResources res;
        BFile file;

        if(file.SetTo(RESOURCEFILEPATH, B_READ_ONLY)==B_OK)
        {
            size_t siz;
            int32 *wk;
            
            res.SetTo(&file);
            
            siz=sizeof(int32);
            wk=(int32 *)res.LoadResource(B_INT32_TYPE,R_FRAME_WIDTH,&siz);
            if(wk!=NULL)
                width=(float)*wk;
            
            siz=sizeof(int32);
            wk=(int32 *)res.LoadResource(B_INT32_TYPE,R_FRAME_HEIGHT,&siz);
            if(wk!=NULL)
                height=(float)*wk;

            file.Unset();    
        }
        
        wndpos.Set(wndpos.left,wndpos.top,
                   wndpos.left+width,wndpos.top+height);
    }
    
    if(wndpos.right>mainscreen.Frame().right)
        wndpos.Set(6,wndpos.top,width,wndpos.bottom);
    if(wndpos.bottom>mainscreen.Frame().bottom)
        wndpos.Set(wndpos.left,be_plain_font->Size()*2+4,
                   wndpos.right,height);

    mainwnd=new BAppMainWindow(wndpos,MAINWINDOW_TITLE); 
    
    EditorList->AddItem((void *)mainwnd); 
    mainwnd->Show(); 

    return mainwnd; 
} 
//--------------------------------------------------------------------- 
void BaseApp::MessageReceived(BMessage *msg) 
{ 
    switch(msg->what) 
    {        
    case MSG_OPEN: 
        open_filepanel->Show(); 
        break; 
    case MSG_QUIT: 
        for(int i=EditorList->CountItems()-1;i>=0;i--) 
            ((BAppMainWindow *)EditorList->ItemAt(i))-> 
                                    PostMessage(B_QUIT_REQUESTED);                            
        break; 
    case MSG_CLOSED: 
        { 
            BAppMainWindow *targetwnd; 
                        
            msg->FindPointer("wnd",(void **)&targetwnd); 
                        
            if(targetwnd!=NULL) 
            {
                targetwnd->Lock(); 
                targetwnd->Quit(); 
                EditorList->RemoveItem(targetwnd); 

                if(EditorList->CountItems()==0) 
                    PostMessage(B_QUIT_REQUESTED); 
            } 
        } 
        break; 
    case MSG_ADDEDITOR: 
        AddEditor(); 
        break;
    case MSG_ACTIVATE:
        { 
            BAppMainWindow *targetwnd; 
                        
            msg->FindPointer("wnd",(void **)&targetwnd); 
                        
            if(targetwnd!=NULL) 
            { 
                EditorList->RemoveItem(targetwnd);
                EditorList->AddItem(targetwnd);
            }
        }
        break; 
    case MSG_SHOWFIND:
        {
            BFindWindow *findwnd=new BFindWindow( 
                BRect(FINDWINDOW_POSITION_LEFT,FINDWINDOW_POSITION_TOP, 
                      FINDWINDOW_POSITION_WIDTH+FINDWINDOW_POSITION_LEFT-1, 
                      FINDWINDOW_POSITION_HEIGHT+FINDWINDOW_POSITION_TOP-1), 
                FINDWINDOW_TITLE,findword,replaceword,ignorecase);
        
            findwnd->Show(); 
        }        
        break;
    case MSG_FIND:
    case MSG_REPLACEALL:
        {
            BAppMainWindow *targetwnd=(BAppMainWindow *)EditorList->
                                    ItemAt(EditorList->CountItems()-1);

            msg->FindString("findword",&findword);
            msg->FindString("replaceword",&replaceword);
            msg->FindBool("ignorecase",&ignorecase);
            if(targetwnd!=NULL)
            {
                targetwnd->Lock();
                targetwnd->Activate();
                if(msg->what==MSG_FIND)
                    targetwnd->FindWord(findword,ignorecase);
                else
                if(msg->what==MSG_REPLACEALL)
                    targetwnd->ReplaceAll(findword,
                                          replaceword,ignorecase);
                targetwnd->Unlock();
            }
        }
        break;
    case MSG_FINDNEXT:
    case MSG_REPLACEFINDNEXT:
    case MSG_REPLACEINSELECTION:
        {
            BAppMainWindow *targetwnd=(BAppMainWindow *)EditorList->
                                    ItemAt(EditorList->CountItems()-1);
            if(findword!="" && targetwnd!=NULL)
            {
                targetwnd->Lock();
                targetwnd->Activate();
                if(msg->what==MSG_REPLACEINSELECTION)
                    targetwnd->ReplaceInSelection(
                                     findword,replaceword,ignorecase);
                else
                {
                    if(msg->what==MSG_REPLACEFINDNEXT)
                        targetwnd->ReplaceWord(replaceword);
                    targetwnd->FindWord(findword,ignorecase);
                }
                targetwnd->Unlock();
            }
        }
        break;
    case MSG_LANGCHANGE:
        {
            usedjapanese=usedjapanese?false:true;
            BAlert *infmsg = new BAlert("Error", 
                                        resmsg(rmLanguageChanged),"OK", 
                                        NULL,NULL,B_WIDTH_AS_USUAL); 
            infmsg->Go(); 
        }
        break;    
    default: 
        BApplication::MessageReceived(msg); 
    } 
} 
//--------------------------------------------------------------------- 
void BaseApp::RefsReceived(BMessage *message) 
{ 
    type_code ref_type; 
    int32 ref_count; 
    entry_ref ref; 
    BEntry entry; 
        
    message->GetInfo("refs", &ref_type,&ref_count); 
    for(int i=0;i<ref_count;i++) 
    { 
        message->FindRef("refs",i,&ref); 
        if(entry.SetTo(&ref)==B_OK) 
        { 
            if(entry.IsFile()) 
            { 
                BAppMainWindow *targetwnd; 

                targetwnd=AddEditor(); 

                targetwnd->Lock(); 
                targetwnd->OpenFile(&entry); 
                targetwnd->Unlock(); 
            } 
        } 
    } 
} 
//--------------------------------------------------------------------- 
void BaseApp::ReadyToRun() 
{ 
    if(EditorList->CountItems()==0) 
        AddEditor(); 
} 
//---------------------------------------------------------------------