//--------------------------------------------------------------------- 
#include "MainWindow.h" 
//--------------------------------------------------------------------- 
BAppMainWindow::BAppMainWindow(BRect frame,const char *title)
    :BWindow(frame,title,MAINWINDOW_WINDOWSTYLE,0) 
{ 
    mainview=new BAppMainView(Bounds());    
    AddChild(mainview); 
    save_filepanel=new BFilePanel(B_SAVE_PANEL); 
    editingpath=NULL; 
    
    fcode=ccASCII;
    fret=rcLF;
    SetCCodeToMenu();

    mainview->memo->MakeFocus(); 
} 
//--------------------------------------------------------------------- 
BAppMainWindow::~BAppMainWindow() 
{ 
    delete save_filepanel; 
    if(editingpath!=NULL) 
        delete editingpath; 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::MessageReceived(BMessage *msg) 
{ 
    switch(msg->what) 
    { 
        case B_UNDO: 
        case B_COPY: 
        case B_CUT: 
        case B_PASTE: 
        case B_SELECT_ALL: 
            mainview->memo->MessageReceived(msg); 
            break; 
        case MSG_SAVE: 
            msg->FindBool("to_quit",&save_to_quit); 
                        
            if(editingpath!=NULL) 
            { 
                BEntry entry; 
                entry.SetTo(editingpath->Path()); 
                SaveFile(&entry); 
                break; 
            } 
        case MSG_SAVEAS: 
            msg->FindBool("to_quit",&save_to_quit); 

            save_filepanel->SetTarget(this); 
            save_filepanel->Show(); 
            break; 
        case B_SAVE_REQUESTED: 
            { 
                entry_ref ref; 
                BString nam; 
                BDirectory dir; 
                BEntry entry; 

                msg->FindRef("directory",&ref); 
                msg->FindString("name",&nam); 
                dir.SetTo(&ref); 

                if(entry.SetTo(&dir,nam.String())==B_OK) 
                { 
                    SaveFile(&entry); 
                    return; 
                } 
            } 
            break;        
        case MSG_CCODECHANGE:
            {
                int32 i;
                msg->FindInt32("code",&i);
                fcode=(enum CCODETYPE)i;
                SetCCodeToMenu();
            }
            break;
        case MSG_CRETCHANGE:
            {
                int32 i;
                msg->FindInt32("ret",&i);
                fret=(enum CRETURNCODE)i;
                SetCCodeToMenu();
            }
            break;
        default: 
            BWindow::MessageReceived(msg); 
    } 
} 
//--------------------------------------------------------------------- 
BAppMainView::BAppMainView(BRect frame) 
    :BView(frame,"bappmainview",B_FOLLOW_ALL,B_WILL_DRAW) 
{ 
    BRect viewrect(Bounds()); 
    BMenuBar *mainmenu=new BMenuBar(BRect(0,0,viewrect.right, 
                                    be_plain_font->Size()*2), 
                                    "mainmenubar"); 
    AddChild(mainmenu); 

    memo=new BMemoView(BRect(0,be_plain_font->Size()*2, 
                             viewrect.right-B_V_SCROLL_BAR_WIDTH, 
                             viewrect.bottom-B_H_SCROLL_BAR_HEIGHT), 
                       "memo",B_FOLLOW_ALL); 
    BScrollView *scr=new BScrollView("memoscroll",memo, 
                                     B_FOLLOW_ALL,0,true,true); 
    AddChild(scr); 

    BMenu *filemenu=new BMenu(resmsg(rmFile)); 
    filemenu->AddItem(new BMenuItem(resmsg(rmOpen),new BMessage(MSG_OPEN),'O')); 
    filemenu->ItemAt(0)->SetTarget(be_app); 
    filemenu->AddItem(new BMenuItem(resmsg(rmNewText),new BMessage(MSG_ADDEDITOR),'N')); 
    filemenu->ItemAt(1)->SetTarget(be_app); 
    filemenu->AddItem(new BMenuItem(resmsg(rmSave),new BMessage(MSG_SAVE),'S')); 
    filemenu->ItemAt(2)->Message()->AddBool("to_quit",false); 
    filemenu->AddItem(new BMenuItem(resmsg(rmSaveAs),new BMessage(MSG_SAVEAS))); 
    filemenu->ItemAt(3)->Message()->AddBool("to_quit",false); 

    savemenu=new BMenu(resmsg(rmCharacterCode));
    savemenu->AddItem(new BMenuItem(resmsg(rmUTF8),new BMessage(MSG_CCODECHANGE)));
    savemenu->ItemAt(0)->Message()->AddInt32("code",ccASCII);
    savemenu->AddItem(new BMenuItem(resmsg(rmSJIS),new BMessage(MSG_CCODECHANGE)));
    savemenu->ItemAt(1)->Message()->AddInt32("code",ccSJIS);
    savemenu->AddItem(new BMenuItem(resmsg(rmJIS),new BMessage(MSG_CCODECHANGE)));
    savemenu->ItemAt(2)->Message()->AddInt32("code",ccJIS);
    savemenu->AddItem(new BMenuItem(resmsg(rmEUC),new BMessage(MSG_CCODECHANGE)));
    savemenu->ItemAt(3)->Message()->AddInt32("code",ccEUC);
    savemenu->AddSeparatorItem(); 
    savemenu->AddItem(new BMenuItem(resmsg(rmLF),new BMessage(MSG_CRETCHANGE)));
    savemenu->ItemAt(5)->Message()->AddInt32("ret",rcLF);
    savemenu->AddItem(new BMenuItem(resmsg(rmCRLF),new BMessage(MSG_CRETCHANGE)));
    savemenu->ItemAt(6)->Message()->AddInt32("ret",rcCRLF);
    filemenu->AddItem(savemenu);

    filemenu->AddSeparatorItem(); 
    filemenu->AddItem(new BMenuItem(resmsg(rmClose),new BMessage(B_QUIT_REQUESTED),'W')); 
    filemenu->AddSeparatorItem(); 
    filemenu->AddItem(new BMenuItem(resmsg(rmQuit),new BMessage(MSG_QUIT),'Q')); 
    filemenu->ItemAt(8)->SetTarget(be_app); 
    mainmenu->AddItem(filemenu); 

    BMenu *editmenu=new BMenu(resmsg(rmEdit)); 
    editmenu->AddItem(new BMenuItem(resmsg(rmUndo),new BMessage(B_UNDO),'Z')); 
    editmenu->AddSeparatorItem(); 
    editmenu->AddItem(new BMenuItem(resmsg(rmCopy),new BMessage(B_COPY),'C')); 
    editmenu->AddItem(new BMenuItem(resmsg(rmCut),new BMessage(B_CUT),'X')); 
    editmenu->AddItem(new BMenuItem(resmsg(rmPaste),new BMessage(B_PASTE),'V')); 
    editmenu->AddItem(new BMenuItem(resmsg(rmSelectAll),new BMessage(B_SELECT_ALL),'A')); 
    mainmenu->AddItem(editmenu); 

    BMenu *searchmenu=new BMenu(resmsg(rmSearch));
    searchmenu->AddItem(new BMenuItem(resmsg(rmFind),new BMessage(MSG_SHOWFIND),'F'));
    searchmenu->AddSeparatorItem(); 
    searchmenu->AddItem(new BMenuItem(resmsg(rmFindNext),new BMessage(MSG_FINDNEXT),'G'));
    searchmenu->AddItem(new BMenuItem(resmsg(rmReplaceFindNext),
                                      new BMessage(MSG_REPLACEFINDNEXT),'R'));
    searchmenu->AddSeparatorItem(); 
    searchmenu->AddItem(new BMenuItem(resmsg(rmReplaceInSelection),
                                      new BMessage(MSG_REPLACEINSELECTION),'E'));
    searchmenu->SetTargetForItems(be_app); 
    mainmenu->AddItem(searchmenu);     

    BMenu *settingmenu=new BMenu(resmsg(rmSetting));
    settingmenu->AddItem(new BMenuItem(resmsg(rmLanguageChange),
                                       new BMessage(MSG_LANGCHANGE)));
    settingmenu->ItemAt(0)->SetTarget(be_app); 
    mainmenu->AddItem(settingmenu);     

    memo->SetDoesUndo(true); 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::OpenFile(BEntry *entry) 
{ 
    if(entry->Exists()) 
    { 
        BFile fil; 
        off_t size; 
        BString str; 
        unsigned char *buf; 

        try 
        { 
            fil.SetTo(entry,B_READ_ONLY | B_FAIL_IF_EXISTS); 
            fil.GetSize(&size); 

            buf=(unsigned char *)malloc(size); 
            fil.ReadAt(0,buf,size); 
            CheckCharCodeType(buf,fcode,fret); 

            if(ccBCType[fcode]!=B_UNICODE_CONVERSION) 
            { 
                int32 buflen,buf2len,state=0; 
                str.SetTo('\0',size*2); 
                char *buf2=str.LockBuffer(size*2); 
                buflen=size; 
                buf2len=size*2; 

                convert_to_utf8(ccBCType[fcode],(char *)buf,&buflen,
                                buf2,&buf2len,&state); 
                str.UnlockBuffer(); 
            } 
            else 
                str.SetTo((char *)buf); 

            if(fret==rcCRLF) 
                str.ReplaceAll("\r\n","\n"); 

            mainview->memo->SetText(str.String()); 

            free(buf); 
            fil.Unset(); 
             
            SetFileName(entry); 
        } 
        catch(...) 
        { 
            BAlert *altmsg = new BAlert("Error", 
                                        resmsg(rmFileReadFailed),"OK", 
                                        NULL,NULL,B_WIDTH_AS_USUAL, 
                                        B_WARNING_ALERT); 
            altmsg->Go(); 
        } 
    } 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::SetFileName(BEntry *entry) 
{ 
    if(editingpath==NULL) 
        editingpath=new BPath(); 
    entry->GetPath(editingpath); 
    
    if(editingpath!=NULL) 
    { 
        char nam[B_FILE_NAME_LENGTH]; 
        
        entry->GetName(nam); 
        SetTitle(nam); 
    } 

    SetCCodeToMenu(); 
    mainview->memo->Modified=false; 
} 
//--------------------------------------------------------------------- 
void BAppMainWindow::SaveFile(BEntry *entry) 
{ 
    BFile fil; 
    off_t wsize; 
    off_t tsize; 
    BString str; 

    try 
    { 
        fil.SetTo(entry,B_READ_WRITE | B_CREATE_FILE); 

        str.SetTo(mainview->memo->Text()); 
            
        if(fret==rcCRLF) 
            str.ReplaceAll("\n","\r\n"); 

        if(ccBCType[fcode]==B_UNICODE_CONVERSION) 
        { 
            tsize=str.Length(); 
            wsize=fil.WriteAt(0,str.String(),tsize); 
        } 
        else 
        { 
            int32 buflen,buf2len,state=0; 

            buf2len=str.Length(); 
            buflen=buf2len*2; 
            char *buf=(char *)malloc(buflen); 
            
            convert_from_utf8(ccBCType[fcode],str.String(),&buf2len,
                              buf,&buflen,&state); 

            tsize=buflen; 
            wsize=fil.WriteAt(0,buf,tsize); 
            free(buf); 
        } 
       
        if(tsize==wsize) 
        { 
            SetFileName(entry); 
            
            if(save_to_quit) 
                PostMessage(new BMessage(B_QUIT_REQUESTED)); 
        } 
    } 
    catch(...) 
    { 
        BAlert *altmsg = new BAlert("Error", resmsg(rmFileWriteFailed),"OK", 
                        NULL,NULL,B_WIDTH_AS_USUAL, 
                        B_WARNING_ALERT); 
        altmsg->Go(); 
    } 
} 
//--------------------------------------------------------------------- 
bool BAppMainWindow::QuitRequested() 
{ 
    bool result; 
    
    result=false; 
    if(mainview->memo!=NULL) 
    { 
        if(mainview->memo->Modified) 
        { 
            char m[1024]; 
            int32 idx; 
            BAlert *wrnmsg; 
            
            sprintf(m,resmsg(rmSaveChangesTo), 
                    (editingpath!=NULL)?editingpath->Leaf():"Untitled"); 

            wrnmsg=new BAlert("Warning!!", m, 
                      resmsg(rmCancel),resmsg(rmDontSave),resmsg(rmSave), 
                              B_WIDTH_AS_USUAL,B_OFFSET_SPACING, 
                              B_WARNING_ALERT); 
            wrnmsg->SetShortcut(0,B_ESCAPE); 
            idx=wrnmsg->Go(); 
            if(idx==0) 
                return false; 
            if(idx==2) 
            { 
                BMessage *msg=new BMessage(MSG_SAVE); 
                msg->AddBool("to_quit",true); 

                PostMessage(msg);                   
                return false; 
            } 
        } 
    } 

	SaveDataToResource();

    BMessage *msg_closed=new BMessage(MSG_CLOSED); 
    msg_closed->AddPointer("wnd",this); 
    be_app->PostMessage(msg_closed); 

    return true; 
} 
//--------------------------------------------------------------------
void BAppMainWindow::WindowActivated(bool active)
{
    if(active)
    {
        BMessage *msg=new BMessage(MSG_ACTIVATE);
        msg->AddPointer("wnd",this);
        be_app->PostMessage(msg);
    }
}
//--------------------------------------------------------------------
void BAppMainWindow::FindWord(const BString &findword,const bool ignorecase)
{
    int32 start,finish,point;
    BString wkstr;

    mainview->memo->GetSelection(&start,&finish);
    wkstr.SetTo(mainview->memo->Text());

    if(ignorecase)
        point=wkstr.IFindFirst(findword,finish);
    else
        point=wkstr.FindFirst(findword,finish);
   
    if(point!=B_ERROR)
        mainview->memo->Select(point,point+findword.Length());
}
//--------------------------------------------------------------------
void BAppMainWindow::ReplaceWord(const BString &replaceword)
{
    int32 start,finish;

    mainview->memo->GetSelection(&start,&finish);
    
    if(start<finish)
    {
        mainview->memo->Delete(start,finish);
        if(replaceword.Length()>0)
            mainview->memo->Insert(start,replaceword.String(),
                                   replaceword.Length());
        mainview->memo->Modified=true;

        mainview->memo->Select(start,start+replaceword.Length());
    }        
}
//--------------------------------------------------------------------
void BAppMainWindow::ReplaceAll(const BString &findword,
                    const BString &replaceword,const bool ignorecase)
{
    int32 old_start,old_finish;

    mainview->memo->GetSelection(&old_start,&old_finish);
    ReplaceWords(findword,replaceword,ignorecase,
                 0,strlen(mainview->memo->Text()));
    mainview->memo->Select(old_start,old_finish);
}
//--------------------------------------------------------------------
void BAppMainWindow::ReplaceInSelection(const BString &findword,
                    const BString &replaceword,const bool ignorecase)
{
    int32 old_start,old_finish;

    mainview->memo->GetSelection(&old_start,&old_finish);

    ReplaceWords(findword,replaceword,ignorecase,
                 old_start,old_finish);
    mainview->memo->Select(old_finish,old_finish);
}
//--------------------------------------------------------------------
void BAppMainWindow::ReplaceWords(const BString &findword,
                    const BString &replaceword,const bool ignorecase,
                    const int32 start,const int32 finish)
{
    int32 point;
    BString wkstr;
    int32 init_length,wkfinish;

    wkstr.SetTo(mainview->memo->Text());
    point=finish;
    wkfinish=finish;
    init_length=wkstr.Length();

    while(point!=B_ERROR && point>=start && point<=wkfinish)
    {
        if(ignorecase)
            point=wkstr.IFindLast(findword,point);
        else
            point=wkstr.FindLast(findword,point);
    
        if(point==B_ERROR)
        {
            wkstr.SetTo(mainview->memo->Text());

            if(ignorecase)
                point=wkstr.IFindFirst(findword);
            else
                point=wkstr.FindFirst(findword);
        }

        if(point!=B_ERROR && point>=start && point<=wkfinish)
        {
            mainview->memo->Select(point,point+findword.Length());
            ReplaceWord(replaceword);
            wkfinish=point;
        }
    }
}
//--------------------------------------------------------------------
void BAppMainWindow::SetCCodeToMenu(void)
{
    const int  MenuIdxCode[7]={0,0,2,1,3,1,0}; //Menu Index
    const int  MenuIdxRet[2]={5,6}; //Menu Index

    for(int i=0;i<mainview->savemenu->CountItems();i++)
        mainview->savemenu->ItemAt(i)->SetMarked(false);

    mainview->savemenu->ItemAt(MenuIdxCode[fcode])->SetMarked(true);
    mainview->savemenu->ItemAt(MenuIdxRet[fret])->SetMarked(true);
}
//--------------------------------------------------------------------
void BAppMainWindow::SaveDataToResource(void)
{
	BResources res;
    BFile file(RESOURCEFILEPATH, B_READ_WRITE | B_CREATE_FILE);
    int32 wk;
    
    res.SetTo(&file);

    wk=Frame().IntegerWidth();
    res.AddResource(B_INT32_TYPE,R_FRAME_WIDTH,&wk,sizeof(wk));
    wk=Frame().IntegerHeight();
    res.AddResource(B_INT32_TYPE,R_FRAME_HEIGHT,&wk,sizeof(wk));

    res.Sync();
    file.Unset();    
}
//--------------------------------------------------------------------